<?php

namespace Drupal\gammu_smsd\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\gammu_smsd\GammuDatabase;
use Symfony\Component\DependencyInjection\ContainerInterface;

class SmsComposeForm extends FormBase {
  /**
   * SmsSentController constructor.
   *
   * @param \Drupal\gammu_smsd\GammuDatabase $gammu_db
   */
  public function __construct(GammuDatabase $gammu_db) {
    $this->gammu_db = $gammu_db;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('gammu_smsd.db')
    );
  }

  public function getFormId() {
    return 'sms_compose_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state, $number = '') {
    $form['gammu_number'] = array(
      '#type' => 'textarea',
      '#title' => t('Your mobile number'),
      '#rows' => 2,
      '#default_value' => $number,
      '#description' => t("Enter your mobile number separated by comma (,)"),
    );
    $form['gammu_phonebook'] = [
      '#type' => 'details',
      '#title' => t('Phone book'),
      '#open' => FALSE,
    ];
    $phonebook = $this->gammu_db->gammu_get_phonebook();

    if (!empty($phonebook)) {
      $form['gammu_phonebook']['search'] = [
        '#type' => 'textfield',
        '#title' => '',
        '#attributes' => [
          'placeholder' => t('Search')
        ],
      ];
      $form['#attached']['library'][] = 'gammu_smsd/gammu_smsd';
      $form['gammu_phonebook']['phone'] = [
        '#type' => 'checkboxes',
        '#title' => t('Check All'),
        '#options' => $phonebook,
      ];
    }
    $form['#tree'] = true;
    $form['gammu_text'] = array(
      '#type' => 'textarea',
      '#title' => t('SMS Text'),
      '#rows' => 3,
      '#default_value' => '',
    );
    $form['actions'] = [
      '#type' => 'actions',
    ];

    // Add a submit button that handles the submission of the form.
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Send SMS'),
    ];
    return $form;
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    $numbers = explode(',', $form_state->getValue("gammu_number"));
    $text = $form_state->getValue("gammu_text");
    if(!empty($form_state->getValue("gammu_phonebook"))){
      $phones = array_filter($form_state->getValue("gammu_phonebook")["phone"]);
    }
    if(!empty($phones)){
      $numbers+= $phones;
    }
    if(!empty($numbers) && !empty($text)){
      $messages = [];
      foreach ($numbers as $number){
        $messages[] = [$number,$text];
      }
      \Drupal::service('gammu_smsd.sendSMS')->send($messages);
    }
  }

}
