<?php

namespace Drupal\gammu_smsd\Form;

use Drupal\Core\Database\Database;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\gammu_smsd\GammuDatabase;
use Symfony\Component\DependencyInjection\ContainerInterface;

class SmsInboxForm extends FormBase {
  /**
   * @var \Drupal\gammu_smsd\GammuDatabase
   */
  protected $gammu_db;

  protected $connection;

  /**
   * SmsSentController constructor.
   *
   * @param \Drupal\gammu_smsd\GammuDatabase $gammu_db
   */
  public function __construct(GammuDatabase $gammu_db) {
    $this->gammu_db = $gammu_db;
    $this->connection = $gammu_db->gammu_connection();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('gammu_smsd.db')
    );
  }
  public function getFormId() {
    return 'sms_inbox_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state) {
    if (!$this->connection) {
      return FALSE;
    }
    $config = $this->config('gammu_smsd.settings');
    $country_code = $config->get('gammu_country_code');
    $data_telephone = $this->gammu_db->gammu_get_phonebook();
    Database::setActiveConnection('gammu_db');
    $sql = "SELECT ID,ReceivingDateTime, SenderNumber , UDH , TextDecoded FROM {inbox} WHERE UDH = '' OR UDH LIKE '%1' ORDER BY ReceivingDateTime DESC";
    $results = $this->connection->query($sql);
    if (!empty($results)) {
      $form['search'] = [
        '#type' => 'search',
        '#title' => '',
        '#attributes' => [
          'placeholder' => t('Search')
        ],
        '#suffix' => '<div id="checkAll"><a href="#checkAll">'.t('Check All').'</a></div>',
      ];
      $form['#attached']['library'][] = 'gammu_smsd/gammu_smsd';

      foreach ($results as $message) {
        $TextDecoded = trim($message->UDH) != '' ? $this->gammu_db->get_message_multipart('inbox', $message->UDH, $message->SenderNumber) : $message->TextDecoded;
        $phone_num = str_replace($country_code, 0, $message->SenderNumber);
        $Number = $country_code.intval($phone_num);
        $phone_name = !empty($data_telephone[$phone_num]) ? $data_telephone[$phone_num] : $phone_num;
        $form[$message->ID] = [
          '#type' => 'checkbox',
          '#default_value' => $message->ID,
          '#title' => $TextDecoded,
          '#description' => $this->gammu_db->gammu_message_detail($Number, $phone_name, $phone_num, $message->ReceivingDateTime),
          '#prefix' => '<div class="list-group-item">',
          '#suffix' => '</div>',
        ];
      }
      $form['actions']['delete'] = [
        '#type' => 'submit',
        '#value' => t('Delete'),
      ];
    }
    Database::setActiveConnection();
    return $form;
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    $ids = array_filter($form_state->getValues());
    $ids = array_intersect_key($ids, array_flip(array_filter(array_keys($ids), 'is_numeric')));
    $this->gammu_db->gammu_delete_message('inbox',array_keys($ids));
  }

}
