<?php

namespace Drupal\gammu_smsd;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Database;
use PDOException;

class GammuDatabase {

  protected \Drupal\Core\Config\ImmutableConfig $config;
  protected $connection;
  /**
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   */

  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->config = $config_factory->get('gammu_smsd.settings');
  }

  public function gammu_connection() {
    $database = Database::getConnection();
    $gammu_db = $database->getConnectionOptions();
    if(!empty($this->config->get('gammu_dbname'))){
      $gammu_db['database'] = $this->config->get('gammu_dbname');
    }
    if(!empty($this->config->get('username'))){
      $gammu_db['username'] = $this->config->get('gammu_dbuser');
    }
    if(!empty($this->config->get('gammu_dbpass'))){
      $gammu_db['password'] = $this->config->get('gammu_dbpass');
    }
    if(!empty($this->config->get('gammu_dbhost'))){
      $gammu_db['host'] = $this->config->get('gammu_dbhost');
    }
    if(!empty($this->config->get('gammu_dbhost'))){
      $gammu_db['driver'] = $this->config->get('gammu_dbengine');
    }
    if(!empty($this->config->get('gammu_dbport'))){
      $gammu_db['port'] = $this->config->get('gammu_dbport');
    }
    // replace 'YourDatabaseKey' with something that's unique to your module
    Database::addConnectionInfo('gammu_db', 'default', $gammu_db);
    try {
      return $this->connection = Database::getConnection('default', 'gammu_db');
    } catch (PDOException $e) {
      \Drupal::messenger()
        ->addError(t("Connection failed, please contact your administrator"));
      return FALSE;
    }
  }

  public function get_message_multipart(string $folder, $UDH, $num_mobile = '') {
    Database::setActiveConnection('gammu_db');
    $udh = substr($UDH, 0, -2);
    $query = $this->connection->select($folder, 'i');
    $query->fields('i', ['TextDecoded']);
    $query->condition('i.UDH', $udh . '%', 'like');
    if ($folder == 'sentitems') {
      $query->condition('i.DestinationNumber', $num_mobile)
        ->orderBy('SequencePosition', 'ASC');
    }
    elseif ($folder == 'inbox') {
      $query->condition('i.UDH', $udh . '%', 'like')->orderBy('ID', 'ASC');
    }
    $results = $query->execute()->fetchCol();
    Database::setActiveConnection();
    return implode('', $results);
  }

  public function gammu_message_detail($Number, $phone_name, $phone_num, $time) {
    include_once \Drupal::root() . '/core/themes/engines/twig/twig.engine';
    return twig_render_template(drupal_get_path('module', 'gammu_smsd') . '/templates/gammu-smsd-message.html.twig', [
      'tel' => $Number,
      'phone_name' => $phone_name,
      'phone_num' => $phone_num,
      'time' => $time,
      'time_format' => $this->gammu_nice_date($time),
      'theme_hook_original' => 'not-applicable',
    ]);
  }

  /*
   * Helper function for gammu
   */
  private function gammu_nice_date($date) {
    $date = strtotime($date);
    $interval = time() - $date;
    //show time ago
    if ($interval < 60 * 60 * 24 * 2) {
      return \Drupal::service('date.formatter')->formatInterval($interval);
    }
    return \Drupal::service('date.formatter')->format($date, 'short');
  }

  public function gammu_getlist_num_phone(string $folder) {
    Database::setActiveConnection('gammu_db');
    $query = $this->connection->select($folder, 'i')->distinct();
    switch ($folder) {
      case  'sentitems':
        $query->fields('i', ['DestinationNumber']);
        break;
      case   'inbox' :
        $query->fields('i', ['SenderNumber']);
        break;
    }
    $or = $query->orConditionGroup()
      ->condition('i.UDH', '')
      ->condition('i.UDH', '%1', 'like');
    $query->condition($or);
    $telephones = $query->execute()->fetchCol();
    Database::setActiveConnection();
    return $telephones;
  }

  public function gammu_get_phonebook() {
    $cid = 'gammu_get_phonebook';
    if ($cache = \Drupal::cache()->get($cid)) {
      return $cache->data;
    }

    $config = $this->config;
    [
      $entity_type,
      $field_name,
      $bundle,
    ] = explode('|', $config->get('gammu_telephone_field'));
    $list = FALSE;

    if (!empty($bundle)) {
      $query = \Drupal::entityQuery($entity_type);
      if (!$config->get("gammu_check_status")) {
        $query->condition('status', TRUE);
      }
      $query->condition('type', $bundle)
        ->condition($field_name, NULL, 'IS NOT NULL');
      $entity_ids = $query->execute();
      if (empty($entity_ids)) {
        return FALSE;
      }
      $name_field = $config->get('gammu_name_field');
      $firstname_field = $config->get('gammu_firstname_field');
      if ($firstname_field || $name_field) {
        foreach ($entity_ids as $id) {
          $bundle_load = \Drupal::entityTypeManager()
            ->getStorage($entity_type)
            ->load($id);
          if (!empty($bundle_load)) {
            $name[] = $this->getNameValue($bundle_load, $name_field);
            if (!empty($firstname_field)) {
              $name[] = $this->getNameValue($bundle_load, $firstname_field);
            }
            $telephones = $bundle_load->get($field_name)->value;
            $name = array_unique($name);
            $name = trim(implode(' ', $name));
            $list[$telephones] = !empty($name) ? $name : $telephones;
            unset($name);
          }
        }
      }
    }
    \Drupal::cache()->set($cid, $list);
    return $list;
  }

  private function getNameValue($bundle_load, $name_field) {
    $field = $bundle_load->get($name_field);
    $definition = $field->getDataDefinition();
    $typePlugin = $definition->getType();
    switch ($typePlugin) {
      case 'address':
        $valueAddress = current($field->getValue());
        if (!empty($valueAddress['family_name'])) {
          $name[] = $valueAddress['family_name'];
        }
        if (!empty($valueAddress['given_name'])) {
          $name[] = $valueAddress['given_name'];
        }
        break;
      case 'name':
        $valueName = current($field->getValue());
        if (!empty($valueName['family'])) {
          $name[] = $valueName['family'];
        }
        if (!empty($valueName['given'])) {
          $name[] = $valueName['given'];
        }
        break;
      default:
        $name[] = $field->value;
        break;
    }
    if (!empty($name)) {
      return implode(' ', $name);
    }
    return '';
  }

  public function gammu_delete_message($folder, $ids) {
    if (is_numeric($ids)) {
      $IDS = [$ids];
    }
    elseif (is_array($ids)) {
      $IDS = $ids;
    }
    Database::setActiveConnection('gammu_db');
    $query_delete = $this->connection->delete($folder);

    if ($folder == 'inbox') {
      foreach ($IDS as $ID) {
        //detect if message is multipart in inbox
        $query = $this->connection->select($folder, 'i');
        $query->fields('i', ['UDH']);
        $query->condition('i.ID', $ID);
        $results = $query->execute()->fetchCol();
        $UDH = end($results);
        if (trim($UDH) != '') {
          $udh = substr($UDH, 0, -2);
          $query_delete->condition('UDH', $udh . '%', 'like');
          $query_delete->execute();
        }
      }
    }
    $query_delete->condition('ID', $IDS,'IN');
    $num_delete = $query_delete->execute();
    Database::setActiveConnection();
    return $num_delete;
  }

}
